# Generated by roxygen2: do not edit by hand

export(angle_btw)
export(angle_deltas)
export(clucenters)
export(clugen)
export(clumerge)
export(clupoints_n)
export(clupoints_n_1)
export(clupoints_n_1_template)
export(clusizes)
export(fix_empty)
export(fix_num_points)
export(llengths)
export(points_on_line)
export(rand_ortho_vector)
export(rand_unit_vector)
export(rand_vector_at_angle)
importFrom(mathjaxr,preview_rd)
