# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

#' clugenr: Multidimensional cluster generation in R
#'
#' clugenr is an R package for generating multidimensional clusters. Each
#' cluster is supported by a line segment, the position, orientation and length
#' of which guide where the respective points are placed.
#'
#' @author Nuno Fachada
#'
#' @note MIT License
#'
#' @docType package
#' @name clugenr
# Suppress R CMD check note
#' @importFrom mathjaxr preview_rd
NULL
#> NULL
