## R CMD check results

0 errors | 0 warnings | 0 notes

## Test environments

* Ubuntu 22.04.2 (R: devel, release, oldrel-1, 3.6.0)
* Windows 11 (R: release)
* macOS 12 (R: release)

## Downstream dependencies

* There are currently no downstream dependencies for this package.

## Other

* Allow direct specification of cluster sizes, cluster centers, line lengths,
  and angle deltas, instead of going through a function.
